@@ -257,6 +257,7 @@ void Expr::GenCaseEval(Output *out_cc, Env *env)
 		(*i)->value()->ForceIDEval(out_cc, env);
 
 	out_cc->println("switch ( %s )", operand_[0]->EvalExpr(out_cc, env));
+	Type* switch_type = operand_[0]->DataType(env);
 
 	out_cc->inc_indent();
 	out_cc->println("{");
@@ -274,7 +275,7 @@ void Expr::GenCaseEval(Output *out_cc, Env *env)
 			}
 		else
 			{
-			GenCaseStr(index, out_cc, env);
+			GenCaseStr(index, out_cc, env, switch_type);
 			out_cc->inc_indent();
 			out_cc->println("%s = %s;", 
 			                env->LValue(val_var),
@@ -285,7 +286,7 @@ void Expr::GenCaseEval(Output *out_cc, Env *env)
 		}
 
 	// Generate the default case after all other cases
-	GenCaseStr(0, out_cc, env);
+	GenCaseStr(0, out_cc, env, switch_type);
 	out_cc->inc_indent();
 	if ( default_case )
 		{
@@ -295,7 +296,7 @@ void Expr::GenCaseEval(Output *out_cc, Env *env)
 		}
 	else
 		{
-		out_cc->println("throw binpac::ExceptionInvalidCaseIndex(\"%s\", %s);",
+		out_cc->println("throw binpac::ExceptionInvalidCaseIndex(\"%s\", (int64)%s);",
 			Location(), operand_[0]->EvalExpr(out_cc, env));
 		}
 	out_cc->println("break;");
@@ -527,6 +528,8 @@ Type *Expr::DataType(Env *env) const
 				{
 				Type *type1 = 
 					cases_->front()->value()->DataType(env);
+				Type* numeric_with_largest_width = 0;
+
 				foreach(i, CaseExprList, cases_)
 					{
 					Type *type2 = 
@@ -540,8 +543,34 @@ Type *Expr::DataType(Env *env) const
 						}
 					if ( type1 == extern_type_nullptr )
 						type1 = type2;
+
+					if ( type2 && type2->IsNumericType() )
+						{
+						if ( numeric_with_largest_width )
+							{
+							int largest;
+							int contender;
+
+							// External C++ types like "int", "bool", "enum" use "int"
+							// storage internally.
+							if ( numeric_with_largest_width->tot() == Type::EXTERN )
+								largest = sizeof(int);
+							else
+								largest = numeric_with_largest_width->StaticSize(env);
+
+							if ( type2->tot() == Type::EXTERN )
+								contender = sizeof(int);
+							else
+								contender = type2->StaticSize(env);
+
+							if ( contender > largest )
+								numeric_with_largest_width = type2;
+							}
+						else
+							numeric_with_largest_width = type2;
+						}
 					}
-				data_type = type1;
+				data_type = numeric_with_largest_width ? numeric_with_largest_width : type1;
 				}
 			else
 				data_type = 0;
